/******************************************************************************
  Copyright (c) 1992 Xerox Corporation.  All rights reserved.
  Portions of this code were written by Stephen White, aka ghond.
  Use and copying of this software and preparation of derivative works based
  upon this software are permitted.  Any distribution of this software or
  derivative works must comply with all applicable United States export
  control laws.  This software is made available AS IS, and Xerox Corporation
  makes no warranty about the software, its performance or its conformity to
  any specification.  Any person obtaining a copy of this software is requested
  to send their name and post office or electronic mail address to:
    Pavel Curtis
    Xerox PARC
    3333 Coyote Hill Rd.
    Palo Alto, CA 94304
    Pavel@Xerox.Com
 *****************************************************************************/

#include "my-stdlib.h"
#include "my-string.h"

#include "config.h"
#include "db.h"
#include "exceptions.h"
#include "structures.h"
#include "match.h"
#include "parse_cmd.h"
#include "storage.h"
#include "unparse.h"
#include "utils.h"

static Var *
aliases(Objid oid)
{
    static Var	answer[2];
    Var         value;
    enum error	e = get_prop_safe(oid, "aliases", &value, 0);

    if (e == E_PROPNF) {
	/* simulate a list of one string */
	answer[0].type = TYPE_NUM;
	answer[0].v.num = 1;
	answer[1].type = TYPE_STR;
	answer[1].v.str = obj_name(oid);
	return answer;
    } else {
	if (value.type == TYPE_LIST)
	    return value.v.list;
	/* simulate an empty list */
	answer[0].type = TYPE_NUM;
	answer[0].v.num = 0;
	return answer;
    } 
}
    

static Objid
match_contents(Objid player, const char *name)
{
    Objid	loc;
    int		step;
    Objid	oid;
    int		lname = strlen(name);
    Objid	exact = NOTHING, partial = FAILED_MATCH;

    if (!valid(player))
	return FAILED_MATCH;
    loc = obj_location(player);

    for (oid = player, step = 0; step < 2; oid = loc, step++) {
	if (!valid(oid))
	    continue;
	for (oid = obj_contents(oid); oid != NOTHING;
	     oid = obj_next_content(oid)) {
	    Var	       *names = aliases(oid);
	    int		i;

	    for (i = 0; i <= names[0].v.num; i++) {
		const char *nm;

		if (i == 0)
		    nm = obj_name(oid);
		else if (names[i].type != TYPE_STR)
		    continue;
		else
		    nm = names[i].v.str;
		
		if (!mystrncasecmp(nm, name, lname)) {
		    if (nm[lname] == '\0') { /* exact match */
			if (exact == NOTHING || exact == oid)
			    exact = oid;
			else
			    return AMBIGUOUS;
		    } else {				/* partial match */
			if (partial == FAILED_MATCH || partial == oid)
			    partial = oid;
			else
			    partial = AMBIGUOUS;
		    }
		}
	    }
	}
    }
    if (exact != NOTHING)
	return exact;
    else
	return partial;
}

Objid
match_object(Objid player, const char *name)
{
    if (name[0] == '\0')
	return NOTHING;
    if (name[0] == '#') {
	char   *p;
	Objid	r = strtol(name + 1, &p, 10);

	if (*p != '\0' || !valid(r))
	    return FAILED_MATCH;
	return r;
    }
    if (!valid(player))
	return FAILED_MATCH;
    if (!mystrcasecmp(name, "me"))
	return player;
    if (!mystrcasecmp(name, "here"))
	return obj_location(player);
    return match_contents(player, name);
}

char rcsid_match[] = "$Id: match.c,v 1.10 1992/10/23 23:03:47 pavel Exp $";

/* $Log: match.c,v $
 * Revision 1.10  1992/10/23  23:03:47  pavel
 * Added copyright notice.
 *
 * Revision 1.9  1992/10/21  03:02:35  pavel
 * Converted to use new automatic configuration system.
 *
 * Revision 1.8  1992/09/14  18:39:47  pjames
 * Updated #includes.  Moved rcsid to bottom.
 *
 * Revision 1.7  1992/09/14  17:43:52  pjames
 * Moved db_modification code to db modules.
 *
 * Revision 1.6  1992/09/03  16:29:24  pjames
 * Changed to use hashing to find property definitions.
 *
 * Revision 1.5  1992/08/31  22:27:12  pjames
 * Changed some `char *'s to `const char *'
 *
 * Revision 1.4  1992/08/21  00:42:43  pavel
 * Renamed include file "parse_command.h" to "parse_cmd.h".
 *
 * Revision 1.3  1992/08/10  16:54:00  pjames
 * Updated #includes.
 *
 * Revision 1.2  1992/07/21  00:04:40  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
